@@ -358,11 +358,6 @@ class CommonAPISession:
         except:
             return unicode(data.decode("utf-8"))
 
-    def _encodeHeaders(self):
-        self.headers = dict((key.encode('ascii') if isinstance(key, unicode) else key,
-                             value.encode('ascii') if isinstance(value, unicode) else value)
-                            for key, value in self.headers.items())
-
     def _sendRequest(self, operation, message, request_headers):
         """ Sending http POST request through URLLIB package
 
@@ -373,7 +368,7 @@ class CommonAPISession:
         :return: responce string data
         """
         operation_url = str(self.url + operation)
-        response = self._http.request('POST', operation_url, body=message.encode('utf-8'), headers=request_headers)
+        response = self._http.request("POST", operation_url, body=message.encode('utf-8'), headers=request_headers)
         return response.data
 
 
@@ -431,30 +426,49 @@ class CommonAPISession:
 
         if 'method_name' not in kwargs:
             raise CloudShellAPIError(404, 'Key "method_name" not in input data!', '')
+
         method_name = kwargs.pop('method_name', None)
 
-        request_node = etree.Element(method_name)
-        #request_str = '<' + method_name + '>\n'
+        message = self._serialize_request(kwargs, method_name)
 
+        result = self._handle_api_response(self._sendRequest(method_name, message))
+
+        return result
+
+    def _handle_api_response(self, response_str):
+        '''
+        :param str response_str:
+        :return: either string or deserialized instance representing the api response
+        '''
+        response_str = self._remove_xml_namespace(response_str)
+        try:
+            api_response = self._deserialize_response(response_str)
+        except Exception:
+            raise Exception("Invalid Request: " + str(response_str))
+        if not api_response.success:
+            raise CloudShellAPIError(api_response.error_code, api_response.error, response_str)
+        result = response_str
+        if api_response.response_info:
+            result = api_response.response_info
+        return result
+
+    def _serialize_request(self, kwargs, method_name):
+        request_node = etree.Element(method_name)
+        # request_str = '<' + method_name + '>\n'
         for name in kwargs:
             child_node = etree.SubElement(request_node, name)
             self._new_serializeRequestData(child_node, kwargs[name])
+        return etree.tostring(request_node).decode("utf-8")
 
-        response_str = self._sendRequest(self.username, self.domain, method_name, etree.tostring(request_node).decode("utf-8"))
-
-        response_str = response_str.replace(b'xmlns="http://schemas.qualisystems.com/ResourceManagement/ApiCommandResult.xsd"', b'') \
-            .replace(b'&#x0;', b'<NUL>')
-
+    def _deserialize_response(self, response_str):
         response_xml = XMLWrapper.parseXML(response_str)
         api_result = CommonApiResult(response_xml)
+        return api_result
 
-        if not api_result.success:
-            raise CloudShellAPIError(api_result.error_code, api_result.error, response_str)
-
-        if api_result.response_info is None:
-            return response_str
-        else:
-            return api_result.response_info
+    def _remove_xml_namespace(self, response_str):
+        return response_str.replace(b'xmlns="http://schemas.qualisystems.com/ResourceManagement/ApiCommandResult.xsd"',
+                                    b'') \
+            .replace(b'&#x0;', b'<NUL>')
 
     def __prettify_xml(self, elem):
         """Return a pretty-printed XML string for the Element.
